// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_ui_components/wyatt_ui_components.dart';

/// Extension on a map of components that creates a [ComponentRegistry]
/// using the map's content.
extension ComponentMapExtension<T extends Component> on Map<String, T> {
  /// Creates and returns a [ComponentRegistry] based on the map's components.
  ComponentRegistry<T> registry() => ComponentRegistry(components: this);
}

/// Extension on a single component that creates a [ComponentRegistry]
/// containing only that component.
extension ComponentExtension<T extends Component> on T {
  /// Creates and returns a [ComponentRegistry] containing
  /// only this component.
  // ignore: use_to_and_as_if_applicable
  ComponentRegistry<T> registry() => ComponentRegistry.solo(this);
}
