// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';
import 'package:wyatt_ui_layout/src/core/scaffold_fields_wrapper.dart';
import 'package:wyatt_ui_layout/src/presentation/layouts/layout.dart';

/// {@template bottom_navigation_bar_layout}
/// A concrete implementation of the [Layout] abstract class for a layout which
/// defines a layout structure with a bottom navigation bar component.
/// {@endtemplate}
class BottomNavigationBarLayout extends StructuralLayout {
  /// {@macro bottom_navigation_bar_layout}
  const BottomNavigationBarLayout({
    required this.body,
    this.custom,
    this.scaffoldFieldsWrapper,
    super.key,
  });

  /// The main content of the layout.
  final Widget? body;

  /// An optional function that can be used to customize the bottom navigation
  /// bar component.
  final BottomNavigationBarComponent? Function(BottomNavigationBarComponent?)?
      custom;

  final ScaffoldFieldsWrapper? scaffoldFieldsWrapper;

  @override
  Widget build(BuildContext context) => Scaffold(
        body: body,
        bottomNavigationBar: custom?.call(
              context.components.bottomNavigationBar,
            ) ??
            context.components.bottomNavigationBar,
        floatingActionButtonLocation:
            scaffoldFieldsWrapper?.floatingActionButtonLocation,
        floatingActionButtonAnimator:
            scaffoldFieldsWrapper?.floatingActionButtonAnimator,
        persistentFooterButtons: scaffoldFieldsWrapper?.persistentFooterButtons,
        persistentFooterAlignment:
            scaffoldFieldsWrapper?.persistentFooterAlignment ??
                AlignmentDirectional.centerEnd,
        drawer: scaffoldFieldsWrapper?.drawer,
        onDrawerChanged: scaffoldFieldsWrapper?.onDrawerChanged,
        endDrawer: scaffoldFieldsWrapper?.endDrawer,
        onEndDrawerChanged: scaffoldFieldsWrapper?.onEndDrawerChanged,
        bottomSheet: scaffoldFieldsWrapper?.bottomSheet,
        backgroundColor: scaffoldFieldsWrapper?.backgroundColor,
        resizeToAvoidBottomInset:
            scaffoldFieldsWrapper?.resizeToAvoidBottomInset,
        primary: scaffoldFieldsWrapper?.primary ?? true,
        drawerDragStartBehavior:
            scaffoldFieldsWrapper?.drawerDragStartBehavior ??
                DragStartBehavior.start,
        extendBody: scaffoldFieldsWrapper?.extendBody ?? false,
        extendBodyBehindAppBar:
            scaffoldFieldsWrapper?.extendBodyBehindAppBar ?? false,
        drawerScrimColor: scaffoldFieldsWrapper?.drawerScrimColor,
        drawerEdgeDragWidth: scaffoldFieldsWrapper?.drawerEdgeDragWidth,
        drawerEnableOpenDragGesture:
            scaffoldFieldsWrapper?.drawerEnableOpenDragGesture ?? true,
        endDrawerEnableOpenDragGesture:
            scaffoldFieldsWrapper?.endDrawerEnableOpenDragGesture ?? true,
        restorationId: scaffoldFieldsWrapper?.restorationId,
      );
}
